From 04c9835b2ccdb519d5cc4dc3167460f4cd172568 Mon Sep 17 00:00:00 2001
From: Olivier Hainque <hainque@adacore.com>
Date: Mon, 7 Mar 2022 10:55:29 +0000
Subject: [PATCH 14/18] Adjust LIBGCC2_INCLUDES for VxWorks and augment comment

Investigating the reasons for libgcc build failures in a canadian
context, orthogonally to the recent update of vxcrtstuff, exposed
interesting differences in the way include search paths are managed
between a regular Linux->VxWorks cross build and a canadian setup
building a Windows->VxWorks toolchain in a Linux environment.

This change augments the comment attached to LIBGCC2_INCLUDE in
libgcc/config/t-vxworks to better describe the parameters at play.

It also adjusts the addition of options for gcc/include and
gcc/include-fixed to minimize the actual differences for libgcc
in the two kinds of configurations.

2022-03-06  Olivier Hainque  <hainque@adacore.com>

libgcc/
	* config/t-vxworks (LIBGCC2_INCLUDE): Augment comment. Move
	-I options for gcc/include and gcc/include-fixed at the end
	and make them -isystem.
---
 libgcc/config/t-vxworks | 36 ++++++++++++++++++++++++++++--------
 1 file changed, 28 insertions(+), 8 deletions(-)

diff --git a/libgcc/config/t-vxworks b/libgcc/config/t-vxworks
index 18a0b1ee469..81a4deab073 100644
--- a/libgcc/config/t-vxworks
+++ b/libgcc/config/t-vxworks
@@ -5,17 +5,37 @@ LIBGCC2_DEBUG_CFLAGS =
 # VxWorks specific entry point.
 LIB2FUNCS_EXCLUDE += _clear_cache
 
-# This ensures that the correct target headers are used; some VxWorks
-# system headers have names that collide with GCC's internal (host)
-# headers, e.g. regs.h. Make sure the local libgcc headers still
-# prevail (e.g. unwind.h), and that gcc provided header files intended
-# to be user visible eventually are visible as well.
+# Arrange to have the correct target headers used when compiling
+# libgcc.  We specifically need to handle VxWorks system headers
+# having names that collide with GCC's internal headers.
+#
+# For example, compiling gthr-vxworks.c #includes <taskLib.h>
+# which in turn #includes <regs.h>, which exists both in the gcc
+# source tree and as a VxWorks system header.  We need that
+# #include to get the VxWorks version, not the compiler one.
+#
+# We still need to make sure that the local libgcc headers prevail
+# (e.g. ./unwind.h), and that gcc provided header files intended
+# to be user visible eventually are visible as well (gcc/include
+# and gcc/include-fixed).
+#
+# The latter are added unconditionally as -isystem during regular
+# cross builds via the %I spec processing and search paths introduced
+# by -B options. These take priority over whatever we can add here,
+# even with -nostdinc, and end up at the tail of the search chain.
+#
+# We keep explicit options nevertheless, to accommodate canadian
+# setups where the libraries are built with an installed cross compiler,
+# without -B.  Also note, incidentally, that the detection of duplicate
+# paths differs between Windows and Linux hosts as the latter can perform
+# inode based checks while the former may only rely on name comparisons.
+
 LIBGCC2_INCLUDES = -nostdinc -I. \
-  -I$(MULTIBUILDTOP)../../gcc/include-fixed$(MULTISUBDIR) \
-  -I$(MULTIBUILDTOP)../../gcc/include \
   $(if $(findstring vxworks7, $(target_noncanonical)), \
     -I$(VSB_DIR)/h -I$(VSB_DIR)/share/h -I=/system -I=/public, \
-    -I=/ -I=/wrn/coreip)
+    -I=/ -I=/wrn/coreip) \
+  -isystem $(MULTIBUILDTOP)../../gcc/include-fixed$(MULTISUBDIR) \
+  -isystem $(MULTIBUILDTOP)../../gcc/include
 
 # Use these also for the vxcrstuff objects (.e.g for version.h), on top of
 # the options possibly already set specifically for the target:
-- 
2.25.1

